from pathlib import Path
from typing import Optional
from seroup.configuration.config import Configuration
from seroup.commands.base import Command
from dataclasses import asdict
import toml


class InitCommand(Command):
    @classmethod
    def name(cls) -> str:
        return "init"

    @classmethod
    def help(cls) -> str:
        return "Creates default sero.toml in specified directory."

    def execute(self, configuration: Optional[Configuration], **kwargs) -> None:
        # ? Get a directory to generate the configuration file in
        # ? or fallback to default
        default_directory = Path.cwd()
        path = kwargs.get("directory", default_directory)

        # ? Generating configuration file path and checking
        # ? if this file already exists
        filepath = path.joinpath("sero.toml")

        if filepath.exists():
            return

        with open(filepath, "w") as file:
            file.write(toml.dumps(asdict(Configuration.default())))



from abc import ABC, abstractmethod
from typing import Optional
from seroup.configuration.config import Configuration


class Command(ABC):
    @classmethod
    @abstractmethod
    def name(cls) -> str:
        pass

    @classmethod
    @abstractmethod
    def help(cls) -> str:
        pass

    @abstractmethod
    def execute(self, configuration: Optional[Configuration], **kwargs) -> None:
        pass



from dataclasses import dataclass, field
from pathlib import Path
from typing import Optional, List, Self
import toml
import dacite
import getpass
import socket


@dataclass
class Server:
    url: str
    subdomain: str

    @classmethod
    def default(cls) -> Self:
        return Server(url="http://sero.com", subdomain=socket.gethostname())


@dataclass
class Cors:
    origins: List[str] = field(default_factory=list)

    @classmethod
    def default(cls) -> Self:
        return Cors(origins=[])


@dataclass
class Credentials:
    login: str
    password: Optional[str]

    @classmethod
    def default(cls) -> Self:
        return Credentials(login=socket.gethostname(), password="")


@dataclass
class Configuration:
    server: Server
    cors: Cors
    credentials: Credentials

    @classmethod
    def from_toml(cls, path: Path) -> Self:
        toml_file_contents: dict = toml.load(path)

        configuration = dacite.from_dict(Configuration, toml_file_contents)

        if not configuration.credentials.password:
            configuration.credentials.password = getpass.getpass()

        return configuration

    @classmethod
    def default(cls) -> Self:
        default_server_configuration = Server.default()
        default_cors_configuration = Cors.default()
        default_credentials_configuration = Credentials.default()

        default_configuration = Configuration(
            server=default_server_configuration,
            cors=default_cors_configuration,
            credentials=default_credentials_configuration,
        )

        return default_configuration



import io
import zipfile
from fnmatch import fnmatch
from pathlib import Path
from typing import Optional

import seroapi as api
from seroup.commands.base import Command
from seroup.configuration.config import Configuration


class UploadCommand(Command):
    @classmethod
    def name(cls) -> str:
        return "upload"

    @classmethod
    def help(cls) -> str:
        return "Upload site."

    def load_ignore_patterns_from_file(self, path: Path) -> list[str]:
        """
        The function reads a file at the specified path and returns a list of patterns excluding empty
        lines and lines starting with "#" symbol. It also strips any leading or trailing whitespace.
        """

        comment_symbol = "#"

        patterns = []

        # ? If path does not exist
        # ? we do not need to ignore anything
        if not path.exists():
            return patterns

        with open(path, "r") as file:
            for line in file:
                line = line.strip()

                if line and not line.startswith(comment_symbol):
                    patterns.append(line)

        return patterns

    def should_ignore(self, path: Path, patterns) -> bool:
        """
        The function `should_ignore` checks if a given path should be ignored based on a list of patterns.

        :param path: The `path` parameter is a Path object that represents a file or directory path in the file system. It is
        used as input to the `should_ignore` method to determine if the path should be ignored based on the provided patterns
        :type path: Path
        :param patterns: The `patterns` parameter is a list of strings that contain wildcard patterns used for matching file
        paths. These patterns are used in the `fnmatch` function to determine if a given file path should be ignored or not
        :return: The `should_ignore` function is returning a boolean value indicating whether the given `path` should be
        ignored based on the `patterns` provided.
        """

        return any(fnmatch(path, pattern) or fnmatch(path.name, pattern) for pattern in patterns)

    def execute(self, configuration: Optional[Configuration], **kwargs):
        # ? Get a directory to generate the configuration file in
        # ? or fallback to default
        default_directory = Path.cwd()
        default_ignore_filename = ".seroignore"
        path = kwargs.get("directory", default_directory)

        ignore_filepath = path.joinpath(default_ignore_filename)

        api_configuration = api.Configuration(host=configuration.server.url)

        # ? Authenticating user
        with api.ApiClient(api_configuration) as api_client:
            instance = api.AccountManagementApi(api_client)

            login = configuration.credentials.login
            password = configuration.credentials.password

            registration_request = api.LoginRequest(login=login, password=password)
            response = instance.login(registration_request)

            api_configuration.access_token = response.token

        with api.ApiClient(api_configuration) as api_client:
            instance = api.ActionsApi(api_client)

            ignore_patterns = self.load_ignore_patterns_from_file(ignore_filepath)

            # ? We should really care about sero.toml
            ignore_patterns.append("sero.toml")

            # ? Compressing to inmemory zip
            # ? excluding root directory
            # ? This also does not compress files which matches
            # ? patterns in .seroignore and sero.toml
            zip_buffer = io.BytesIO()

            with zipfile.ZipFile(zip_buffer, "w", zipfile.ZIP_DEFLATED) as zipf:
                for file_path in path.glob("./**/*"):
                    if file_path.is_dir():
                        continue  # Skip directories

                    if self.should_ignore(file_path, ignore_patterns):
                        continue  # Skip ignored files

                    relative_path = file_path.relative_to(path)
                    zipf.write(file_path, relative_path)
            zip_buffer.seek(0)

            subdomain = configuration.server.subdomain
            instance.upload_site(x_subdomain=subdomain, archive=zip_buffer.read())



from pathlib import Path
from typing import Optional
from seroup.configuration.config import Configuration
from seroup.commands.base import Command
import seroapi as api


class DownloadCommand(Command):
    @classmethod
    def name(cls) -> str:
        return "download"

    @classmethod
    def help(cls) -> str:
        return "Download site."

    def execute(self, configuration: Optional[Configuration], **kwargs) -> None:
        # ? Get a directory to generate the configuration file in
        # ? or fallback to default
        default_directory = Path.cwd()
        path = kwargs.get("directory", default_directory)

        # ? Generate filepath for zip archive
        # ? which will be downloaded
        filename = path.joinpath(f"{configuration.server.subdomain}.zip")

        api_configuration = api.Configuration(host=configuration.server.url)

        # ? Authenticating user
        with api.ApiClient(api_configuration) as api_client:
            instance = api.AccountManagementApi(api_client)

            login = configuration.credentials.login
            password = configuration.credentials.password

            registration_request = api.LoginRequest(login=login, password=password)
            response = instance.login(registration_request)

            api_configuration.access_token = response.token

        # ? Call download action
        with api.ApiClient(api_configuration) as api_client:
            instance = api.ActionsApi(api_client)
            response = instance.download_site_without_preload_content(x_subdomain=configuration.server.subdomain)

            with open(filename, "wb") as f:
                f.write(response.read())



from typing import Optional
from seroup.configuration.config import Configuration
from seroup.commands.base import Command
import seroapi as api


class EnableCommand(Command):
    @classmethod
    def name(cls) -> str:
        return "enable"

    @classmethod
    def help(cls) -> str:
        return "Enables site."

    def execute(self, configuration: Optional[Configuration], **kwargs) -> None:
        api_configuration = api.Configuration(host=configuration.server.url)

        # ? Authenticating user
        with api.ApiClient(api_configuration) as api_client:
            instance = api.AccountManagementApi(api_client)

            login = configuration.credentials.login
            password = configuration.credentials.password

            registration_request = api.LoginRequest(login=login, password=password)
            response = instance.login(registration_request)

            api_configuration.access_token = response.token

        # ? Call enable action
        with api.ApiClient(api_configuration) as api_client:
            instance = api.ActionsApi(api_client)
            instance.enable_site(x_subdomain=configuration.server.subdomain)
